/* -*- mode: c++; c-basic-offset: 3; -*- */

#include "frtoc_times.hh"
#include "FrameF.hh"
#include <iostream>

using namespace std;

//======================================  Open FrameF constructor
frtoc_times::frtoc_times(FrameF& ffin) {
   build_skeleton(ffin);
}

//======================================  Open frame stream constructor
frtoc_times::frtoc_times(std::istream& in) {
   FrameF ffin(in);
   build_skeleton(ffin);
}

//======================================  Get data from TOC
void
frtoc_times::build_skeleton(FrameF& ffin) {
   frame_vector.clear();

   //----------------------------------  Position at nominal start of TOC
   ffin.ReadHeader();
   ffin.Seek(-20, ios::end);
   long seekToc = ffin.getLong();
   if (!seekToc) return;
   ffin.Seek(-seekToc, ios::end);
   
   //----------------------------------- Skip past FrSH and FrSE structures
   while (ffin.NxStruct()) {
      int id = ffin.getID();
      if (id != 1 && id != 2) break;
   }

   //-----------------------------------  Get the id
   ffin.getShort();
   size_t nFrames = ffin.getInt();
   ffin.Skip(4*nFrames);                // skip the data quality words

   //-----------------------------------  Fill the times
   frame_vector.resize(nFrames);
   for (size_t i=0; i<nFrames; i++) {
      frame_vector[i].first.setS(ffin.getInt());
   }
   
   for (size_t i=0; i<nFrames; i++) {
      frame_vector[i].first.setN(ffin.getInt());
   }

   for (size_t i=0; i<nFrames; i++) {
      frame_vector[i].second = Interval(ffin.getDouble());
   }
}

Interval
frtoc_times::duration(void) const {
   if (frame_vector.empty()) return Interval(0.0);

   size_t N = frame_vector.size();
   if (N == 1) return frame_vector[0].second;

   return frame_vector[N-1].first + frame_vector[N-1].second - start();
}
